@@ -102,6 +102,24 @@ class TestConfig(base.TestCase):
         self.assertEqual('123456789', cc.auth['project_domain_id'])
         self.assertNotIn('domain_id', cc.auth)
         self.assertNotIn('domain-id', cc.auth)
+        self.assertNotIn('domain_id', cc)
+
+    def test_get_one_cloud_domain_scoped(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cc = c.get_one_cloud('_test-cloud-domain-scoped_')
+        self.assertEqual('12345', cc.auth['domain_id'])
+        self.assertNotIn('user_domain_id', cc.auth)
+        self.assertNotIn('project_domain_id', cc.auth)
+
+    def test_get_one_cloud_infer_user_domain(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cc = c.get_one_cloud('_test-cloud-int-project_')
+        self.assertEqual('awesome-domain', cc.auth['user_domain_id'])
+        self.assertEqual('awesome-domain', cc.auth['project_domain_id'])
+        self.assertNotIn('domain_id', cc.auth)
+        self.assertNotIn('domain_id', cc)
 
     def test_get_one_cloud_with_hyphenated_project_id(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -171,6 +189,34 @@ class TestConfig(base.TestCase):
         self.assertEqual('user', cc.auth['username'])
         self.assertEqual('testpass', cc.auth['password'])
 
+    def test_get_one_cloud_networks(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cc = c.get_one_cloud('_test-cloud-networks_')
+        self.assertEqual(
+            ['a-public', 'another-public', 'split-default'],
+            cc.get_external_networks())
+        self.assertEqual(
+            ['a-private', 'another-private', 'split-no-default'],
+            cc.get_internal_networks())
+        self.assertEqual('another-private', cc.get_nat_destination())
+        self.assertEqual('another-public', cc.get_default_network())
+        self.assertEqual(
+            ['a-public', 'another-public', 'split-no-default'],
+            cc.get_external_ipv4_networks())
+        self.assertEqual(
+            ['a-public', 'another-public', 'split-default'],
+            cc.get_external_ipv6_networks())
+
+    def test_get_one_cloud_no_networks(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cc = c.get_one_cloud('_test-cloud-domain-scoped_')
+        self.assertEqual([], cc.get_external_networks())
+        self.assertEqual([], cc.get_internal_networks())
+        self.assertIsNone(cc.get_nat_destination())
+        self.assertIsNone(cc.get_default_network())
+
     def test_only_secure_yaml(self):
         c = config.OpenStackConfig(config_files=['nonexistent'],
                                    vendor_files=['nonexistent'],
@@ -183,7 +229,9 @@ class TestConfig(base.TestCase):
                                    secure_files=[self.no_yaml])
         self.assertEqual(
             ['_test-cloud-domain-id_',
+             '_test-cloud-domain-scoped_',
              '_test-cloud-int-project_',
+             '_test-cloud-networks_',
              '_test-cloud_',
              '_test-cloud_no_region',
              '_test_cloud_hyphenated',
@@ -335,6 +383,59 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
 
+    def test_get_one_cloud_precedence(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+
+        kwargs = {
+            'auth': {
+                'username': 'testuser',
+                'password': 'authpass',
+                'project-id': 'testproject',
+                'auth_url': 'http://example.com/v2',
+            },
+            'region_name': 'kwarg_region',
+            'password': 'ansible_password',
+            'arbitrary': 'value',
+        }
+
+        args = dict(
+            auth_url='http://example.com/v2',
+            username='user',
+            password='argpass',
+            project_name='project',
+            region_name='region2',
+            snack_type='cookie',
+        )
+
+        options = argparse.Namespace(**args)
+        cc = c.get_one_cloud(
+            argparse=options, **kwargs)
+        self.assertEqual(cc.region_name, 'region2')
+        self.assertEqual(cc.auth['password'], 'argpass')
+        self.assertEqual(cc.snack_type, 'cookie')
+
+    def test_get_one_cloud_precedence_no_argparse(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+
+        kwargs = {
+            'auth': {
+                'username': 'testuser',
+                'password': 'authpass',
+                'project-id': 'testproject',
+                'auth_url': 'http://example.com/v2',
+            },
+            'region_name': 'kwarg_region',
+            'password': 'ansible_password',
+            'arbitrary': 'value',
+        }
+
+        cc = c.get_one_cloud(**kwargs)
+        self.assertEqual(cc.region_name, 'kwarg_region')
+        self.assertEqual(cc.auth['password'], 'authpass')
+        self.assertIsNone(cc.password)
+
     def test_get_one_cloud_just_argparse(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
@@ -696,3 +797,108 @@ class TestBackwardsCompatibility(base.TestCase):
             'auth_type': 'v3password',
         }
         self.assertDictEqual(expected, result)
+
+    def test_project_v2password(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cloud = {
+            'auth_type': 'v2password',
+            'auth': {
+                'project-name': 'my_project_name',
+                'project-id': 'my_project_id'
+            }
+        }
+        result = c._fix_backwards_project(cloud)
+        expected = {
+            'auth_type': 'v2password',
+            'auth': {
+                'tenant_name': 'my_project_name',
+                'tenant_id': 'my_project_id'
+            }
+        }
+        self.assertEqual(expected, result)
+
+    def test_project_password(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cloud = {
+            'auth_type': 'password',
+            'auth': {
+                'project-name': 'my_project_name',
+                'project-id': 'my_project_id'
+            }
+        }
+        result = c._fix_backwards_project(cloud)
+        expected = {
+            'auth_type': 'password',
+            'auth': {
+                'project_name': 'my_project_name',
+                'project_id': 'my_project_id'
+            }
+        }
+        self.assertEqual(expected, result)
+
+    def test_backwards_network_fail(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cloud = {
+            'external_network': 'public',
+            'networks': [
+                {'name': 'private', 'routes_externally': False},
+            ]
+        }
+        self.assertRaises(
+            exceptions.OpenStackConfigException,
+            c._fix_backwards_networks, cloud)
+
+    def test_backwards_network(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cloud = {
+            'external_network': 'public',
+            'internal_network': 'private',
+        }
+        result = c._fix_backwards_networks(cloud)
+        expected = {
+            'external_network': 'public',
+            'internal_network': 'private',
+            'networks': [
+                {'name': 'public', 'routes_externally': True,
+                 'nat_destination': False, 'default_interface': True},
+                {'name': 'private', 'routes_externally': False,
+                 'nat_destination': True, 'default_interface': False},
+            ]
+        }
+        self.assertEqual(expected, result)
+
+    def test_normalize_network(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cloud = {
+            'networks': [
+                {'name': 'private'}
+            ]
+        }
+        result = c._fix_backwards_networks(cloud)
+        expected = {
+            'networks': [
+                {'name': 'private', 'routes_externally': False,
+                 'nat_destination': False, 'default_interface': False,
+                 'routes_ipv4_externally': False,
+                 'routes_ipv6_externally': False},
+            ]
+        }
+        self.assertEqual(expected, result)
+
+    def test_single_default_interface(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        cloud = {
+            'networks': [
+                {'name': 'blue', 'default_interface': True},
+                {'name': 'purple', 'default_interface': True},
+            ]
+        }
+        self.assertRaises(
+            exceptions.OpenStackConfigException,
+            c._fix_backwards_networks, cloud)
